#include <stdlib.h>

#include <sstream>
#include <iomanip>

#include "core/dss.h" 

#include "core/model/apartment.h"
#include "core/model/device.h"
#include "core/model/group.h"
#include "core/model/modelconst.h"
#include "core/model/modelevent.h"
#include "core/model/modelmaintenance.h"
#include "core/model/modulator.h"
#include "core/model/set.h"
#include "core/model/zone.h"

#include "core/base.h"
#include "core/datetools.h" 
#include "core/businterface.h"
#include "core/ds485/dsbusinterface.h"
#include "core/ds485/dsmeteringbusinterface.h"

#include "core/web/plugin/webserver_plugin.h" 

using namespace dss;

int plugin_getversion() {
	return WEBSERVER_PLUGIN_API_VERSION;
} // plugin_getversion


bool dataRequest(DSS& _dss, HashMapConstStringString& _parameter,
		std::string& result) {
	std::stringstream sStream;
	std::vector< boost::shared_ptr<DSMeter> > modulators = _dss.getApartment().getDSMeters();
	std::vector< boost::shared_ptr<Zone> > zones = _dss.getApartment().getZones();
	
	sStream << "{\"Name\":\""<<_dss.getApartment().getName() <<"\",";
	sStream << "\"Zone\":[";

	bool firstZone = true;

        for (size_t i = 0; i < zones.size();i++) {
                boost::shared_ptr<Zone> ipZone = zones.at(i);

		if (firstZone) {
			firstZone = false;
		} else {
			sStream << ",";
		}
		sStream << "{\"Name\":\""<< ipZone->getName() << "\",";
		sStream << "\"ID\":"<< ipZone->getID() << ",";
		sStream << "\"Groups\":[";

		bool firstGroup = true;
		std::vector< boost::shared_ptr<Group> > groups = ipZone->getGroups();
                for (size_t g = 0; g < groups.size(); g++) {
                        boost::shared_ptr<Group> ipGroup = groups.at(g);
			if (firstGroup) {
				firstGroup = false;
			} else {
				sStream << ",";
			}
			sStream << "{\"lastScene\":"<< ipGroup->getLastCalledScene() << ",";
			
			Set pSet=ipGroup->getDevices();
			if (ipGroup->getID() == 0)
				sStream << "\"DeviceCount\":0,";
			else
				sStream << "\"DeviceCount\":" << pSet.length() << ",";
			sStream << "\"Name\":\"" << ipGroup->getName() << "\",";
			sStream << "\"ID\":"<< ipGroup->getID() << "}";

		}
		sStream << "]}";
	}

	sStream << "],\"Circuit\":[ ";

	bool firstModulator = true;
        for (size_t m = 0; m < modulators.size(); m++) {
                boost::shared_ptr<DSMeter> ipModulator = modulators.at(m);
		if (firstModulator) {
			firstModulator = false;
		} else {
			sStream << ",";
		}

		sStream << "{\"Name\":\""<< ipModulator->getName() <<"\",";
		sStream << "\"dSID\":\"" << ipModulator->getDSID().toString() << "\",";
		sStream << "\"HardwareVersion\":\"" << ipModulator->getHardwareVersion() << "\",";
		sStream << "\"SoftwareVersion\":\"" << ipModulator->getArmSoftwareVersion() << "\",";
		sStream << "\"SoftwareDSPVersion\":\"" << ipModulator->getDspSoftwareVersion() << "\",";
		sStream << "\"orangeEnergyLevel\":" << ipModulator->getEnergyLevelOrange() << ",";
		sStream << "\"redEnergyLevel\":" << ipModulator->getEnergyLevelRed() << "";
		sStream << "}";
	}
	sStream << "]}";

	result = sStream.str();
	return true;
}

bool steadyCallRequest(DSS& _dss, HashMapConstStringString& _parameter,
		std::string& result) {
	bool energyAnfrage=strToIntDef(_parameter["parameter"], -1) & 0x01;
	bool lastSceneAnfrage=strToIntDef(_parameter["parameter"], -1) & 0x02;
	bool firstModulator;

    std::stringstream sStream;
	sStream << "{";
	if (energyAnfrage) 
	{
		// send energy metering request on the bus
		DSS::getInstance()->getBusInterface().getMeteringBusInterface()->requestMeterData();

		std::vector<boost::shared_ptr<DSMeter> > modulators = _dss.getApartment().getDSMeters();

		sStream << "\"Energy\":[";

		firstModulator = true;
                for (size_t m = 0; m < modulators.size(); m++) {
                        boost::shared_ptr<DSMeter> ipModulator = modulators.at(m);
			if (firstModulator) {
				firstModulator = false;
			} else {
				sStream << ",";
			}
			sStream << "{\"dSID\":\""<< ipModulator->getDSID().toString() <<"\",";
			sStream << "\"value\":" << ipModulator->getCachedPowerConsumption() << "}";
		}
		
		sStream << "]";
	}
	if (energyAnfrage && lastSceneAnfrage) 
	{
		sStream << ",";
	}
	if (lastSceneAnfrage) 
	{
		sStream << "\"lastScene\":[";
		
		std::vector<boost::shared_ptr<Zone> > zones = _dss.getApartment().getZones();		
		bool firstZone = true;
                for (size_t i = 0; i < zones.size();i++) {
                        boost::shared_ptr<Zone> ipZone = zones.at(i); 
			if (firstZone) {
				firstZone = false;
			} else {
				sStream << ",";
			}
			sStream << "{\"ID\":"<< ipZone->getID() << ",";
			sStream << "\"Groups\":[";

			bool firstGroup = true;
			std::vector< boost::shared_ptr<Group> > groups = ipZone->getGroups();
                        for (size_t g = 0; g < groups.size(); g++) {
                                boost::shared_ptr<Group> ipGroup = groups.at(g);
				if (firstGroup) {
					firstGroup = false;
				} else {
					sStream << ",";
				}
				sStream << "{\"lastScene\":"<< ipGroup->getLastCalledScene() << ",";
				sStream << "\"ID\":"<< ipGroup->getID() << "}";

			}
			sStream << "]}";
		}
		
		sStream << "]";
	}
	sStream << "}";
	result = sStream.str();
	
	return true;
}

bool actionRequest(DSS& _dss, HashMapConstStringString& _parameter,
		std::string& result) {
	int iZoneID=strToIntDef(_parameter["zoneID"], -1);
	int iGroupID=strToIntDef(_parameter["groupID"], -1);
	int iSzeneID=strToIntDef(_parameter["szeneID"], -1);
	if (iSzeneID==61)
		iSzeneID=32;
	else if (iSzeneID==62)
		iSzeneID=31;
	else
		switch (iSzeneID)
		{
			case 40:
				iSzeneID=64;
				break;
			case 43:
				iSzeneID=65;
				break;
			case 44:
				iSzeneID=66;
				break;
			case 48:
				iSzeneID=67;
				break;
			case 49:
				iSzeneID=68;
				break;
			case 58:
				iSzeneID=69;
				break;
			case 59:
				iSzeneID=70;
				break;
			case 55:
				iSzeneID=71;
				break;
			case 56:
				iSzeneID=72;
				break;
			case 41:
				iSzeneID=73;
				break;
			case 42:
				iSzeneID=74;
				break;
			default:
				break;
		}
	
	
	boost::shared_ptr<Zone> pZone = _dss.getApartment().getZone(iZoneID);
	boost::shared_ptr<Group> pGroup = pZone->getGroup(iGroupID);
	pGroup->callScene(iSzeneID);
	result = "";
	return true;	
}

bool dimmUpRequest(DSS& _dss, HashMapConstStringString& _parameter,
		std::string& result) {
	int iZoneID=strToIntDef(_parameter["zoneID"], -1);
	int iGroupID=strToIntDef(_parameter["groupID"], -1);
	boost::shared_ptr<Zone> pZone = _dss.getApartment().getZone(iZoneID);
	boost::shared_ptr<Group> pGroup = pZone->getGroup(iGroupID);
	pGroup->increaseValue();
	result = "";
	return true;	
}

bool dimmDownRequest(DSS& _dss, HashMapConstStringString& _parameter,
		std::string& result) {
	int iZoneID=strToIntDef(_parameter["zoneID"], -1);
	int iGroupID=strToIntDef(_parameter["groupID"], -1);
	boost::shared_ptr<Zone> pZone = _dss.getApartment().getZone(iZoneID);
	boost::shared_ptr<Group> pGroup = pZone->getGroup(iGroupID);
	pGroup->decreaseValue();
	result = "";
	return true;	
}

bool saveRequest(DSS& _dss, HashMapConstStringString& _parameter,
		std::string& result) {
	int iZoneID=strToIntDef(_parameter["zoneID"], -1);
	int iGroupID=strToIntDef(_parameter["groupID"], -1);
	int iSzeneID=strToIntDef(_parameter["szeneID"], -1);
	boost::shared_ptr<Zone> pZone = _dss.getApartment().getZone(iZoneID);
	boost::shared_ptr<Group> pGroup = pZone->getGroup(iGroupID);
	pGroup->saveScene(iSzeneID);
	result = "";
	return true;
}

bool plugin_handlerequest(const std::string& _uri,
		HashMapConstStringString& _parameter, DSS& _dss,
		std::string& result) {

	if (_uri == "/iPhone/data") {
		return dataRequest(_dss, _parameter, result);
	}
	if (_uri == "/iPhone/steadyCall") {
		return steadyCallRequest(_dss, _parameter, result);
	}
	if (_uri == "/iPhone/action") {
		return actionRequest(_dss, _parameter, result);
	}
	if (_uri == "/iPhone/dimmUp") {
		return dimmUpRequest(_dss, _parameter, result);
	}
	if (_uri == "/iPhone/dimmDown") {
		return dimmDownRequest(_dss, _parameter, result);
	}
	if (_uri == "/iPhone/saveScene") {
		return saveRequest(_dss, _parameter, result);
	}
	return false;
} // plugin_handlerequest

